@@ -64,7 +64,7 @@ macro_rules! uint_full_mul_reg {
         $crate::uint_full_mul_reg!($name, 8, $self_expr, $other, |a, b| a != 0 || b != 0);
     };
     ($name:ident, $n_words:tt, $self_expr:expr, $other:expr) => {
-        uint_full_mul_reg!($name, $n_words, $self_expr, $other, |_, _| true);
+        uint_full_mul_reg!($name, $n_words, $self_expr, $other, |_, _| true)
     };
     ($name:ident, $n_words:tt, $self_expr:expr, $other:expr, $check:expr) => {{
         {
