@@ -125,24 +125,27 @@
 
 use core::cmp::Ord;
 use core::convert::{TryFrom, TryInto};
-use core::str::FromStr;
 use core::{fmt, i64, marker::PhantomData};
 
 use typenum::Unsigned;
 
+use crate::string::Stringify;
+
 #[cfg(feature = "i128")]
-use crate::i256::I256;
-use crate::ops::*;
+use i256::I256;
+use ops::*;
 pub use typenum;
 
 mod const_fn;
 mod errors;
+mod float;
 #[cfg(feature = "i128")]
 mod i256;
 mod macros;
 #[cfg(feature = "parity")]
 mod parity;
 mod power_table;
+mod string;
 #[cfg(test)]
 mod tests;
 
@@ -152,8 +155,8 @@ compile_error!("Some of the next features must be enabled: \"i128\", \"i64\", \"
 pub use errors::*;
 
 pub mod ops;
-#[cfg(all(feature = "serde", feature = "std"))]
-#[cfg_attr(docsrs, doc(cfg(all(feature = "std", feature = "serde"))))]
+#[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
 pub mod serde;
 
 #[doc(hidden)]
@@ -504,6 +507,7 @@ macro_rules! impl_fixed_point {
             }
 
             #[inline]
+            #[must_use]
             pub fn round_towards_zero_by(
                 self,
                 precision: FixedPoint<$layout, P>,
@@ -573,35 +577,18 @@ macro_rules! impl_fixed_point {
         $(#[$attr])?
         impl<P: Precision> fmt::Debug for FixedPoint<$layout, P> {
             fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(f, "{}", self)
+                let mut buf = Default::default();
+                self.stringify(&mut buf);
+                f.write_str(buf.as_str())
             }
         }
 
         $(#[$attr])?
         impl<P: Precision> fmt::Display for FixedPoint<$layout, P> {
             fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                let sign = self.inner.signum();
-                let integral = (self.inner / Self::COEF).abs();
-                let mut fractional = (self.inner % Self::COEF).abs();
-                let mut frac_width = if fractional > 0 {
-                    Self::PRECISION as usize
-                } else {
-                    0
-                };
-
-                while fractional > 0 && fractional % 10 == 0 {
-                    fractional /= 10;
-                    frac_width -= 1;
-                }
-
-                write!(
-                    f,
-                    "{}{}.{:0width$}",
-                    if sign < 0 { "-" } else { "" },
-                    integral,
-                    fractional,
-                    width = frac_width
-                )
+                let mut buf = Default::default();
+                self.stringify(&mut buf);
+                f.write_str(buf.as_str())
             }
         }
 
@@ -634,38 +621,6 @@ macro_rules! impl_fixed_point {
             }
         }
 
-        #[cfg(feature = "std")]
-        impl<P: Precision> TryFrom<f64> for FixedPoint<$layout, P> {
-            type Error = ConvertError;
-
-            // TODO: it's a baseline implementation. See #18.
-            fn try_from(value: f64) -> Result<Self, Self::Error> {
-                use std::io::Write;
-
-                if !value.is_finite() {
-                    return Err(ConvertError::new("not finite"));
-                }
-
-                let mut buffer = [0u8; 64];
-
-                let abs = value.abs();
-
-                // f64 seems to have at least 15 significant decimal digits.
-                let prec = 15usize.saturating_sub(abs.log10().ceil() as usize)
-                    .min(Self::PRECISION as usize)
-                    .max(0);
-
-                write!(&mut buffer[..], "{:.*}", prec, value)
-                    .map_err(|_| ConvertError::new("too big number"))?;
-
-                let s = std::str::from_utf8(&buffer)
-                    .map_err(|_| ConvertError::new("not finite"))?
-                    .trim_end_matches('\0');
-
-                s.parse().map_err(|_| ConvertError::new("not finite"))
-            }
-        }
-
         $(
             // TODO: how to make the repetition replacement trick with `$(#[$attr])`?
             impl<P: Precision> TryFrom<$try_from> for FixedPoint<$layout, P> {
@@ -680,62 +635,6 @@ macro_rules! impl_fixed_point {
                 }
             }
         )*
-
-        $(#[$attr])?
-        impl<P: Precision> FromStr for FixedPoint<$layout, P> {
-            type Err = ConvertError;
-
-            fn from_str(str: &str) -> Result<Self, Self::Err> {
-                let str = str.trim();
-                let coef = Self::COEF;
-
-                let index = match str.find('.') {
-                    Some(index) => index,
-                    None => {
-                        let integral: $layout = str.parse().map_err(|_| {
-                            ConvertError::new("can't parse integral part of the str")
-                        })?;
-                        return integral
-                            .checked_mul(coef)
-                            .ok_or(ConvertError::new("too big integral part"))
-                            .map(Self::from_bits);
-                    }
-                };
-
-                let integral: $layout = str[0..index]
-                    .parse()
-                    .map_err(|_| ConvertError::new("can't parse integral part"))?;
-                let fractional_str = &str[index + 1..];
-
-                if !fractional_str.chars().all(|c| c.is_digit(10)) {
-                    return Err(ConvertError::new("can't parse fractional part: must contain digits only"));
-                }
-
-                if fractional_str.len() > Self::PRECISION.abs() as usize {
-                    return Err(ConvertError::new("requested precision is too high"));
-                }
-
-                let ten: $layout = 10;
-                let exp = ten.pow(fractional_str.len() as u32);
-
-                if exp > coef {
-                    return Err(ConvertError::new("requested precision is too high"));
-                }
-
-                let fractional: $layout = fractional_str.parse().map_err(|_| {
-                    ConvertError::new("can't parse fractional part")
-                })?;
-
-                let final_integral = integral.checked_mul(coef).ok_or(ConvertError::new("too big integral"))?;
-                let signum = if str.as_bytes()[0] == b'-' { -1 } else { 1 };
-                let final_fractional = signum * coef / exp * fractional;
-
-                final_integral
-                    .checked_add(final_fractional)
-                    .map(Self::from_bits)
-                    .ok_or(ConvertError::new("too big number"))
-            }
-        }
     };
 }
 
