@@ -1,4 +1,8 @@
-#[cfg(feature = "std")]
+#[cfg(not(feature = "std"))]
+extern crate alloc;
+#[cfg(not(feature = "std"))]
+use alloc::format;
+
 use core::f64;
 use core::i64;
 
@@ -27,17 +31,17 @@ fn from_decimal() -> Result<()> {
 }
 
 #[test]
-#[cfg(feature = "std")]
 fn display() -> Result<()> {
     test_fixed_point! {
         case (x | FixedPoint, expected | &str) => {
-            assert_eq!(format!("{}", x), String::from(expected));
+            assert_eq!(format!("{}", x), expected);
 
             #[cfg(feature = "serde")]
             assert_eq!(serde_json::to_string(&x).unwrap(), format!("\"{}\"", expected));
         },
         all {
             (fp!(0), "0.0");
+            (fp!(42), "42.0");
             (fp!(10.042), "10.042");
             (fp!(-10.042), "-10.042");
             (fp!(0.000000001), "0.000000001");
@@ -64,7 +68,7 @@ fn from_good_str() -> Result<()> {
             let input: FixedPoint = input.parse()?;
             assert_eq!(input, expected);
 
-            #[cfg(all(feature = "std", feature = "serde"))]
+            #[cfg(feature = "serde")]
             assert_eq!(
                 serde_json::from_str::<FixedPoint>(&format!("\"{}\"", input)).unwrap(),
                 expected
@@ -109,7 +113,7 @@ fn from_bad_str() -> Result<()> {
             let result: Result<FixedPoint, ConvertError> = bad_str.parse();
             assert!(result.is_err(), "must not parse '{}'", bad_str);
 
-            #[cfg(all(feature = "std", feature = "serde"))]
+            #[cfg(feature = "serde")]
             assert!(serde_json::from_str::<FixedPoint>(&format!("\"{}\"", bad_str)).is_err());
         },
         all {
@@ -134,7 +138,7 @@ fn from_bad_str() -> Result<()> {
 }
 
 #[test]
-#[cfg(all(feature = "std", feature = "serde"))]
+#[cfg(feature = "serde")]
 fn serde_with() -> Result<()> {
     test_fixed_point! {
         case (input | f64, expected | FixedPoint) => {
@@ -692,7 +696,6 @@ fn rounding_to_i64() -> Result<()> {
 }
 
 #[test]
-#[cfg(feature = "std")]
 #[allow(clippy::float_cmp)]
 fn to_f64() -> Result<()> {
     test_fixed_point! {
@@ -731,13 +734,11 @@ fn to_f64() -> Result<()> {
 }
 
 #[test]
-#[cfg(feature = "std")]
-#[allow(clippy::float_cmp)]
 fn from_f64() -> Result<()> {
     test_fixed_point! {
-        case (x | FixedPoint, expected | f64) => {
-            assert_eq!(Ok(x), expected.try_into());
-            assert_eq!(Ok(x.cneg().unwrap()), (-expected).try_into());
+        case (expected | FixedPoint, x | f64) => {
+            assert_eq!(FixedPoint::try_from(x)?, expected);
+            assert_eq!(FixedPoint::try_from(-x)?, expected.cneg()?);
         },
         all {
             (fp!(0), 0.0);
@@ -775,6 +776,55 @@ fn from_f64() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn from_f64_exact() -> Result<()> {
+    test_fixed_point! {
+        case (x | f64, expected | FixedPoint) => {
+            assert_eq!(FixedPoint::try_from(x)?, expected);
+            assert_eq!(FixedPoint::try_from(-x)?, expected.cneg()?);
+        },
+        all {
+            (f64::MIN_POSITIVE, fp!(0));
+            (1e-19, fp!(0));
+            (0.000006927_f64, fp!(0.000006927));
+            (0.00006927_f64, fp!(0.00006927));
+            (0.1_f64, fp!(0.1));
+            (0.12345_f64, fp!(0.12345));
+            (0.6927_f64, fp!(0.6927));
+            (5.1_f64, fp!(5.1));
+            (1643804666.060961, fp!(1643804666.060961));
+        },
+        fp64 {
+            (f64::EPSILON, fp!(0));
+            (0.123_456_789_412_345_61_f64, fp!(0.123456789));
+            (0.123_456_789_512_345_61_f64, fp!(0.12345679));
+        },
+        fp128 {
+            (f64::EPSILON, fp!(0.000000000000000222));
+            (0.123_456_789_012_345_61_f64, fp!(0.1234567890123456));
+            (0.123_456_789_012_345_68_f64, fp!(0.1234567890123457));
+        },
+    };
+    Ok(())
+}
+
+#[test]
+fn from_f64_limits() -> Result<()> {
+    test_fixed_point! {
+        case (x | f64, expected | ConvertError) => {
+            assert_eq!(FixedPoint::try_from(x), Err(expected));
+        },
+        all {
+            (f64::NAN, ConvertError::new("not finite"));
+            (f64::INFINITY, ConvertError::new("not finite"));
+            (f64::NEG_INFINITY, ConvertError::new("not finite"));
+            (f64::MAX, ConvertError::new("too big number"));
+            (f64::MIN, ConvertError::new("too big number"));
+        },
+    };
+    Ok(())
+}
+
 #[test]
 fn saturating_add() -> Result<()> {
     test_fixed_point! {
